<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Form\DataMapper;

use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Dedi\SyliusQuotePlugin\Modifier\QuoteItemQuantityModifierInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Symfony\Component\Form\Extension\Core\DataMapper\PropertyPathMapper;
use Symfony\Component\Form\FormInterface;
use Traversable;

class QuoteItemQuantityDataMapper implements QuoteItemQuantityDataMapperInterface
{
    private PropertyPathMapper $propertyPathDataMapper;

    private ChannelContextInterface $channelContext;

    private QuoteItemQuantityModifierInterface $quoteItemQuantityModifier;

    public function __construct(
        PropertyPathMapper $propertyPathDataMapper,
        ChannelContextInterface $channelContext,
        QuoteItemQuantityModifierInterface $quoteItemQuantityModifier
    ) {
        $this->propertyPathDataMapper = $propertyPathDataMapper;
        $this->channelContext = $channelContext;
        $this->quoteItemQuantityModifier = $quoteItemQuantityModifier;
    }

    public function mapDataToForms($viewData, $forms): void
    {
        $this->propertyPathDataMapper->mapDataToForms($viewData, $forms);
    }

    /**
     * @param FormInterface[]|Traversable $forms
     * @param QuoteItemInterface $viewData
     */
    public function mapFormsToData($forms, &$viewData): void
    {
        $formsOtherThanQuantity = [];
        foreach ($forms as $form) {
            if ('quantity' === $form->getName()) {
                $targetQuantity = $form->getData();
                $this->quoteItemQuantityModifier->modify($viewData, (int) $targetQuantity);

                continue;
            }

            $formsOtherThanQuantity[] = $form;
        }

        if (!empty($formsOtherThanQuantity)) {
            $this->propertyPathDataMapper->mapFormsToData($formsOtherThanQuantity, $viewData);
        }
    }
}
